@@ -1,5 +1,6 @@
+# MIT License
 #
-# __COPYRIGHT__
+# Copyright The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -19,12 +20,9 @@
 # LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-#
-__revision__ = "__FILE__ __REVISION__ __DATE__ __DEVELOPER__"
 
 import SCons.compat
 
-import copy
 import sys
 import unittest
 
@@ -867,10 +865,9 @@ class TaskmasterTestCase(unittest.TestCase):
         except MyException as e:
             exc_caught = 1
             exc_value = e
-        except Exception as e:
-            exc_actually_caught = e
+        except Exception as exc_actually_caught:
             pass
-        assert exc_caught, "did not catch expected MyException: %s"%exc_actually_caught
+        assert exc_caught, "did not catch expected MyException: %s" % exc_actually_caught
         assert str(exc_value) == "exception value", exc_value
         assert built_text is None, built_text
 
