@@ -1,5 +1,6 @@
+# MIT License
 #
-# __COPYRIGHT__
+# Copyright The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -19,12 +20,8 @@
 # LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-#
-
-__revision__ = "__FILE__ __REVISION__ __DATE__ __DEVELOPER__"
 
 import os.path
-import sys
 import unittest
 import fnmatch
 
@@ -97,7 +94,7 @@ public class Foo
         """Test class names with $ in them"""
 
         input = """\
-public class BadDep { 
+public class BadDep {
   public void new$rand () {}
 }
 """
@@ -484,6 +481,43 @@ public class NestedExample
         expect = [ 'NestedExample$1', 'NestedExample$1$1', 'NestedExample' ]
         assert expect == classes, (expect, classes)
 
+    def test_lambda_after_new(self):
+        """Test lamdas after new"""
+
+        input = """\
+// import java.util.*;
+
+public class LamdaExample
+{
+
+    public void testFunc (int arg1, String arg2, Runnable lambda){
+    }
+    public LamdaExample()
+    {
+        testFunc(
+            5,
+            new String("test"),
+            // Lambda symbol is after new, and used curly braces so
+            // we should not parse this as a new class.
+            () -> {}
+        );
+    }
+
+
+    public static void main(String argv[])
+    {
+            LamdaExample e = new LamdaExample();
+    }
+}
+"""
+        pkg_dir, classes = SCons.Tool.JavaCommon.parse_java(input, '1.4')
+        expect = [ 'LamdaExample' ]
+        assert expect == classes, (expect, classes)
+
+        pkg_dir, classes = SCons.Tool.JavaCommon.parse_java(input, '1.8')
+        expect = [ 'LamdaExample' ]
+        assert expect == classes, (expect, classes)
+
     def test_private_inner_class_instantiation(self):
         """Test anonymous inner class generated by private instantiation"""
 
@@ -532,7 +566,7 @@ class Broken
    * Detected.
    */
   class InnerOK { InnerOK () { } }
-  
+
   {
     System.out.println("a number: " + 1000.0 + "");
   }
@@ -638,24 +672,43 @@ public class AnonDemo {
         Verify that the java path globs work with specific examples.
         :return:
         """
-        from SCons.Tool.JavaCommon import java_linux_include_dirs_glob, java_linux_version_include_dirs_glob, java_win32_dir_glob, java_win32_version_dir_glob, java_macos_include_dir_glob, java_macos_version_include_dir_glob
+        from SCons.Tool.JavaCommon import (
+            java_linux_include_dirs_glob,
+            java_linux_version_include_dirs_glob,
+            java_win32_dir_glob,
+            java_win32_version_dir_glob,
+            java_macos_include_dir_glob,
+            java_macos_version_include_dir_glob,
+        )
 
         # Test windows globs
         win_java_dirs = [
-            ('C:/Program Files/Java/jdk1.8.0_201/bin', '1.8.0'),
-            ('C:/Program Files/Java/jdk-11.0.2/bin', '11.0.2'),
-            ('C:/Program Files/Java/jdk1.7.0_80/bin', '1.7.0')
+            (r'C:/Program Files/Java/jdk1.8.0_201/bin', '1.8.0'),
+            (r'C:/Program Files/Java/jdk-11.0.2/bin', '11.0.2'),
+            (r'C:/Program Files/AdoptOpenJDK/jdk-16.0.1.9-hotspot/bin', '16.0.1'),
+            (r'C:/Program Files/Microsoft/jdk-17.0.0.35-hotspot/bin', '17.0.0'),
+            (r'C:/Program Files/OpenJDK/openjdk-11.0.13_8/bin', '11.0.13'),
+            (r'C:/Program Files/RedHat/java-1.8.0-openjdk-1.8.0.312-1/bin', '1.8.0'),
+            (r'C:/Program Files/RedHat/java-11-openjdk-11.0.13-1/bin', '11.0.13'),
         ]
 
         for (wjd, version) in win_java_dirs:
             if not fnmatch.fnmatch(wjd, java_win32_dir_glob):
-                self.fail("Didn't properly match %s with pattern %s" % (wjd, java_win32_dir_glob))
+                self.fail(
+                    "Didn't properly match %s with pattern %s"
+                    % (wjd, java_win32_dir_glob)
+                )
             if not fnmatch.fnmatch(wjd, java_win32_version_dir_glob % version):
-                self.fail("Didn't properly match %s with version (%s) specific pattern %s" % (
-                wjd, version, java_win32_version_dir_glob % version))
+                self.fail(
+                    "Didn't properly match %s with version (%s) specific pattern %s"
+                    % (wjd, version, java_win32_version_dir_glob % version)
+                )
 
         non_win_java_include_dirs = [
-            ('/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.191.b12-0.el7_5.x86_64/include', '1.8.0'),
+            (
+                '/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.191.b12-0.el7_5.x86_64/include',
+                '1.8.0',
+            ),
             ('/usr/lib/jvm/java-1.8.0-openjdk-amd64/include', '1.8.0'),
             ('/usr/lib/jvm/java-8-openjdk-amd64/include', '8'),
         ]
@@ -663,7 +716,7 @@ public class AnonDemo {
         # Test non-windows/non-macos globs
         for (wjd, version) in non_win_java_include_dirs:
             match = False
-            globs_tried =[]
+            globs_tried = []
             for jlig in java_linux_include_dirs_glob:
                 globs_tried.append(jlig)
 
@@ -672,36 +725,52 @@ public class AnonDemo {
                     break
 
             if not match:
-                self.fail("Didn't properly match %s with pattern %s" % (wjd, globs_tried))
+                self.fail(
+                    "Didn't properly match %s with pattern %s" % (wjd, globs_tried)
+                )
 
             match = False
             globs_tried = []
             for jlvig in java_linux_version_include_dirs_glob:
-                globs_tried.append(jlvig%version)
+                globs_tried.append(jlvig % version)
                 if fnmatch.fnmatch(wjd, jlvig % version):
                     match = True
                     break
 
             if not match:
-                self.fail("Didn't properly match %s with version (%s) specific pattern %s" % (
-                    wjd, version, globs_tried))
+                self.fail(
+                    "Didn't properly match %s with version (%s) specific pattern %s"
+                    % (wjd, version, globs_tried)
+                )
 
         # Test macos globs
         # Test windows globs
         macos_java_dirs = [
             # ('/System/Library/Frameworks/JavaVM.framework/Headers/', None),
-            ('/System/Library/Frameworks/JavaVM.framework/Versions/11.0.2/Headers/', '11.0.2'),
+            (
+                '/System/Library/Frameworks/JavaVM.framework/Versions/11.0.2/Headers/',
+                '11.0.2',
+            ),
         ]
 
-        if not fnmatch.fnmatch('/System/Library/Frameworks/JavaVM.framework/Headers/', java_macos_include_dir_glob):
-            self.fail("Didn't properly match %s with pattern %s" % ('/System/Library/Frameworks/JavaVM.framework/Headers/', java_macos_include_dir_glob))
+        if not fnmatch.fnmatch(
+            '/System/Library/Frameworks/JavaVM.framework/Headers/',
+            java_macos_include_dir_glob,
+        ):
+            self.fail(
+                "Didn't properly match %s with pattern %s"
+                % (
+                    '/System/Library/Frameworks/JavaVM.framework/Headers/',
+                    java_macos_include_dir_glob,
+                )
+            )
 
         for (wjd, version) in macos_java_dirs:
             if not fnmatch.fnmatch(wjd, java_macos_version_include_dir_glob % version):
-                self.fail("Didn't properly match %s with version (%s) specific pattern %s" % (
-                wjd, version, java_macos_version_include_dir_glob % version))
-
-
+                self.fail(
+                    "Didn't properly match %s with version (%s) specific pattern %s"
+                    % (wjd, version, java_macos_version_include_dir_glob % version)
+                )
 
 
 if __name__ == "__main__":
