@@ -1,5 +1,5 @@
 """
-TestCmd.py:  a testing framework for commands and scripts.
+A testing framework for commands and scripts.
 
 The TestCmd module provides a framework for portable automated testing
 of executable commands and scripts (in any language, not just Python),
@@ -299,9 +299,12 @@ __version__ = "1.3"
 import atexit
 import difflib
 import errno
+import hashlib
 import os
 import re
+import psutil
 import shutil
+import signal
 import stat
 import subprocess
 import sys
@@ -309,20 +312,20 @@ import tempfile
 import threading
 import time
 import traceback
-import types
 from collections import UserList, UserString
+from pathlib import Path
 from subprocess import PIPE, STDOUT
-
+from typing import Optional
 
 IS_WINDOWS = sys.platform == 'win32'
+IS_MACOS = sys.platform == 'darwin'
 IS_64_BIT = sys.maxsize > 2**32
 IS_PYPY = hasattr(sys, 'pypy_translation_info')
-
+NEED_HELPER = os.environ.get('SCONS_NO_DIRECT_SCRIPT')
 
 # sentinel for cases where None won't do
 _Null = object()
 
-
 __all__ = [
     'diff_re',
     'fail_test',
@@ -345,13 +348,13 @@ def is_List(e):
 
 
 def to_bytes(s):
-    if isinstance(s, bytes) or bytes is str:
+    if isinstance(s, bytes):
         return s
     return bytes(s, 'utf-8')
 
 
 def to_str(s):
-    if bytes is str or is_String(s):
+    if is_String(s):
         return s
     return str(s, 'utf-8')
 
@@ -384,17 +387,57 @@ def _caller(tblist, skip):
     return string
 
 
-def fail_test(self=None, condition=True, function=None, skip=0, message=None):
-    """Cause the test to fail.
+def clean_up_ninja_daemon(self, result_type):
+    """
+    Kill any running scons daemon started by ninja and clean up it's working dir and
+    temp files.
+    """
+    if self:
+        for path in Path(self.workdir).rglob('.ninja'):
+            daemon_dir = Path(tempfile.gettempdir()) / (
+                "scons_daemon_" + str(hashlib.md5(str(path.resolve()).encode()).hexdigest())
+            )
+            pidfiles = [daemon_dir / 'pidfile', path / 'scons_daemon_dirty']
+            for pidfile in pidfiles:
+                if pidfile.exists():
+                    with open(pidfile) as f:
+                        try:
+                            pid = int(f.read())
+                            os.kill(pid, signal.SIGINT)
+                        except OSError:
+                            pass
+                    
+                        while True:
+                            if pid not in [proc.pid for proc in psutil.process_iter()]:
+                                break
+                            else:
+                                time.sleep(0.1)
+                
+            if not self._preserve[result_type]:
+                if daemon_dir.exists():
+                    shutil.rmtree(daemon_dir)
 
-    By default, the fail_test() method reports that the test FAILED
-    and exits with a status of 1.  If a condition argument is supplied,
-    the test fails only if the condition is true.
+
+def fail_test(self=None, condition=True, function=None, skip=0, message=None):
+    """Causes a test to exit with a fail.
+
+    Reports that the test FAILED and exits with a status of 1, unless
+    a condition argument is supplied; if so the completion processing
+    takes place only if the condition is true.
+
+    Args:
+        self: a test class instance. Must be passed in explicitly
+            by the caller since this is an unbound method.
+        condition (optional): if false, return to let test continue.
+        function (optional): function to call before completion processing.
+        skip (optional): how many lines at the top of the traceback to skip.
+        message (optional): additional text to include in the fail message.
     """
     if not condition:
         return
     if function is not None:
         function()
+    clean_up_ninja_daemon(self, 'fail_test')
     of = ""
     desc = ""
     sep = " "
@@ -417,16 +460,30 @@ def fail_test(self=None, condition=True, function=None, skip=0, message=None):
 
 
 def no_result(self=None, condition=True, function=None, skip=0):
-    """Causes a test to exit with no valid result.
+    """Causes a test to exit with a no result.
 
-    By default, the no_result() method reports NO RESULT for the test
-    and exits with a status of 2.  If a condition argument is supplied,
-    the test fails only if the condition is true.
+    In testing parlance NO RESULT means the test could not be completed
+    for reasons that imply neither success nor failure - for example a
+    component needed to run the test could be found. However, at this
+    point we still have an "outcome", so record the information and exit
+    with a status code of 2, unless a condition argument is supplied;
+    if so the completion processing takes place only if the condition is true.
+
+    The different exit code and message allows other logic to distinguish
+    from a fail and decide how to treat NO RESULT tests.
+
+    Args:
+        self: a test class instance. Must be passed in explicitly
+            by the caller since this is an unbound method.
+        condition (optional): if false, return to let test continue.
+        function (optional): function to call before completion processing.
+        skip (optional): how many lines at the top of the traceback to skip.
     """
     if not condition:
         return
     if function is not None:
         function()
+    clean_up_ninja_daemon(self, 'no_result')
     of = ""
     desc = ""
     sep = " "
@@ -445,16 +502,25 @@ def no_result(self=None, condition=True, function=None, skip=0):
 
 
 def pass_test(self=None, condition=True, function=None):
-    """Causes a test to pass.
+    """Causes a test to exit with a pass.
+
+    Reports that the test PASSED and exits with a status of 0, unless
+    a condition argument is supplied; if so the completion processing
+    takes place only if the condition is true.
 
-    By default, the pass_test() method reports PASSED for the test
-    and exits with a status of 0.  If a condition argument is supplied,
     the test passes only if the condition is true.
+
+    Args:
+        self: a test class instance. Must be passed in explicitly
+            by the caller since this is an unbound method.
+        condition (optional): if false, return to let test continue.
+        function (optional): function to call before completion processing.
     """
     if not condition:
         return
     if function is not None:
         function()
+    clean_up_ninja_daemon(self, 'pass_test')
     sys.stderr.write("PASSED\n")
     sys.exit(0)
 
@@ -467,10 +533,10 @@ def match_exact(lines=None, matches=None, newline=os.sep):
     :param matches: expected lines to match
     :type matches: str or list[str]
     :param newline: line separator
-    :returns: an object (1) on match, else None, like re.match
+    :returns: None on failure, 1 on success.
 
     """
-    if isinstance(lines, bytes) or bytes is str:
+    if isinstance(lines, bytes):
         newline = to_bytes(newline)
 
     if not is_List(lines):
@@ -496,8 +562,7 @@ def match_caseinsensitive(lines=None, matches=None):
     :type lines: str or list[str]
     :param matches: expected lines to match
     :type matches: str or list[str]
-    :returns: True or False
-    :returns: an object (1) on match, else None, like re.match
+    :returns: None on failure, 1 on success.
 
     """
     if not is_List(lines):
@@ -519,7 +584,7 @@ def match_re(lines=None, res=None):
     :type lines: str or list[str]
     :param res: regular expression(s) for matching
     :type res: str or list[str]
-    :returns: an object (1) on match, else None, like re.match
+    :returns: None on failure, 1 on success.
 
     """
     if not is_List(lines):
@@ -554,7 +619,7 @@ def match_re_dotall(lines=None, res=None):
     :type lines: str or list[str]
     :param res: regular expression(s) for matching
     :type res: str or list[str]
-    :returns: a match object, or None as for re.match
+    :returns: a match object on match, else None, like re.match
 
     """
     if not isinstance(lines, str):
@@ -575,15 +640,13 @@ def simple_diff(a, b, fromfile='', tofile='',
     r"""Compare two sequences of lines; generate the delta as a simple diff.
 
     Similar to difflib.context_diff and difflib.unified_diff but
-    output is like from the 'diff" command without arguments. The function
+    output is like from the "diff" command without arguments. The function
     keeps the same signature as the difflib ones so they will be
     interchangeable,  but except for lineterm, the arguments beyond the
     two sequences are ignored in this version. By default, the
     diff is not created with trailing newlines, set the lineterm
     argument to '\n' to do so.
 
-    :raises re.error: if a regex fails to compile
-
     Example:
 
     >>> print(''.join(simple_diff('one\ntwo\nthree\nfour\n'.splitlines(True),
@@ -624,13 +687,16 @@ def simple_diff(a, b, fromfile='', tofile='',
 
 def diff_re(a, b, fromfile='', tofile='',
             fromfiledate='', tofiledate='', n=3, lineterm='\n'):
-    """Compare a and b (lists of strings) where a are regexes.
+    """Compare a and b (lists of strings) where a are regular expressions.
 
     A simple "diff" of two sets of lines when the expected lines
     are regular expressions.  This is a really dumb thing that
     just compares each line in turn, so it doesn't look for
     chunks of matching lines and the like--but at least it lets
     you know exactly which line first didn't compare correctl...
+
+    Raises:
+        re.error: if a regex fails to compile
     """
     result = []
     diff = len(a) - len(b)
@@ -1097,6 +1163,9 @@ class TestCmd:
                 interpreter = [interpreter]
             cmd = list(interpreter) + cmd
         if arguments:
+            if isinstance(arguments, dict):
+                cmd.extend(["%s=%s" % (k, v) for k, v in arguments.items()])
+                return cmd
             if isinstance(arguments, str):
                 arguments = arguments.split()
             cmd.extend(arguments)
@@ -1208,8 +1277,7 @@ class TestCmd:
         return match_stderr_function(lines, matches)
 
     def match_stdout(self, lines, matches):
-        """Compare actual and expected file contents.
-        """
+        """Compare actual and expected file contents."""
         try:
             match_stdout_function = getattr(self, self._match_stdout_function)
         except TypeError:
@@ -1238,8 +1306,7 @@ class TestCmd:
                   skip=skip)
 
     def pass_test(self, condition=True, function=None):
-        """Cause the test to pass.
-        """
+        """Cause the test to pass."""
         if not condition:
             return
         self.condition = 'pass_test'
@@ -1543,6 +1610,9 @@ class TestCmd:
 
         The specified program will have the original directory
         prepended unless it is enclosed in a [list].
+
+        argument: If this is a dict() then will create arguments with KEY+VALUE for
+                  each entry in the dict.
         """
         if self.external:
             if not program:
@@ -1616,7 +1686,7 @@ class TestCmd:
         """
         time.sleep(seconds)
 
-    def stderr(self, run=None):
+    def stderr(self, run=None) -> Optional[str]:
         """Returns the error output from the specified run number.
 
         If there is no specified run number, then returns the error
@@ -1628,10 +1698,13 @@ class TestCmd:
             run = len(self._stderr)
         elif run < 0:
             run = len(self._stderr) + run
-        run = run - 1
-        return self._stderr[run]
+        run -= 1
+        try:
+            return self._stderr[run]
+        except IndexError:
+            return None
 
-    def stdout(self, run=None):
+    def stdout(self, run=None) -> Optional[str]:
         """Returns the stored standard output from a given run.
 
         Args:
@@ -1648,7 +1721,7 @@ class TestCmd:
             run = len(self._stdout)
         elif run < 0:
             run = len(self._stdout) + run
-        run = run - 1
+        run -= 1
         try:
             return self._stdout[run]
         except IndexError:
