@@ -1,6 +1,28 @@
+# MIT License
+#
+# Copyright The SCons Foundation
+#
+# Permission is hereby granted, free of charge, to any person obtaining
+# a copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
+#
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
+# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
+# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 """
-TestSCons.py:  a testing framework for the SCons software construction
-tool.
+A testing framework for the SCons software construction tool.
 
 A TestSCons environment object is created via the usual invocation:
 
@@ -12,20 +34,18 @@ from those classes, as well as any overridden or additional methods or
 attributes defined in this subclass.
 """
 
-# __COPYRIGHT__
-
-__revision__ = "__FILE__ __REVISION__ __DATE__ __DEVELOPER__"
-
 import os
 import re
 import shutil
 import sys
 import time
-import subprocess
+import subprocess as sp
+import zipfile
 from collections import namedtuple
 
 from TestCommon import *
 from TestCommon import __all__
+from SCons.Util import get_hash_format, get_current_hash_algorithm_used
 
 from TestCmd import Popen
 from TestCmd import PIPE
@@ -35,10 +55,12 @@ from TestCmd import PIPE
 # here provides some independent verification that what we packaged
 # conforms to what we expect.
 
-default_version = '4.1.0.devyyyymmdd'
+default_version = '4.3.1ayyyymmdd'
 
-python_version_unsupported = (3, 4, 0)
-python_version_deprecated = (3, 4, 0)
+# TODO: these need to be hand-edited when there are changes
+python_version_unsupported = (3, 6, 0)
+python_version_deprecated = (3, 6, 0)
+python_version_supported_str = "3.6.0"  # str of lowest non-deprecated version
 
 # In the checked-in source, the value of SConsVersion in the following
 # line must remain "__ VERSION __" (without the spaces) so the built
@@ -157,11 +179,13 @@ def deprecated_python_version(version=sys.version_info):
 
 if deprecated_python_version():
     msg = r"""
-scons: warning: Support for pre-2.7.0 Python version (%s) is deprecated.
+scons: warning: Support for pre-%s Python version (%s) is deprecated.
     If this will cause hardship, contact scons-dev@scons.org
 """
-
-    deprecated_python_expr = re_escape(msg % python_version_string()) + file_expr
+    deprecated_python_expr = (
+        re_escape(msg % (python_version_supported_str, python_version_string()))
+        + file_expr
+    )
     del msg
 else:
     deprecated_python_expr = ""
@@ -188,7 +212,7 @@ def initialize_sconsflags(ignore_python_version):
     # support the --warn=no-visual-c-missing warning.)
     visual_c = os.environ.get('TESTSCONS_SCONSFLAGS',
                               '--warn=no-visual-c-missing')
-    if visual_c:
+    if visual_c and visual_c not in sconsflags:
         sconsflags.append(visual_c)
     os.environ['SCONSFLAGS'] = ' '.join(sconsflags)
     return save_sconsflags
@@ -283,8 +307,6 @@ class TestSCons(TestCommon):
                 kw['program'] = os.path.join(self.orig_cwd, kw['program'])
         if 'interpreter' not in kw and not os.environ.get('SCONS_EXEC'):
             kw['interpreter'] = [python, ]
-            if sys.version_info[0] < 3:
-                kw['interpreter'].append('-tt')
         if 'match' not in kw:
             kw['match'] = match_exact
         if 'workdir' not in kw:
@@ -299,7 +321,7 @@ class TestSCons(TestCommon):
         if kw.get('ignore_python_version', -1) != -1:
             del kw['ignore_python_version']
 
-        TestCommon.__init__(self, **kw)
+        super().__init__(**kw)
 
         if not self.external:
             import SCons.Node.FS
@@ -400,20 +422,35 @@ class TestSCons(TestCommon):
 
         return None
 
-    def wrap_stdout(self, build_str="", read_str="", error=0, cleaning=0):
-        """Wraps standard output string(s) in the normal
-        "Reading ... done" and "Building ... done" strings
+    def wrap_stdout(self, build_str="", read_str="", error=0, cleaning=0) -> str:
+        """Wraps "expect" strings in SCons boilerplate.
+
+        Given strings of expected output specific to a test,
+        returns a string which includes the SCons wrapping such as
+        "Reading ... done", etc.: that is, adds the text that would
+        be left out by running SCons in quiet mode;
+        Makes a complete message to match against.
+
+        Args:
+            read_str: the message for the execution part of the output.
+                If non-empty, needs to be newline-terminated.
+            read_str: the message for the reading-sconscript part of
+                the output. If non-empty, needs to be newline-terminated.
+            error: if true, expect a fail message rather than a done message.
+            cleaning (int): index into type messages, if 0 selects
+                build messages, if 1 selects clean messages.
         """
         cap, lc = [('Build', 'build'),
                    ('Clean', 'clean')][cleaning]
         if error:
-            term = "scons: %sing terminated because of errors.\n" % lc
+            term = f"scons: {lc}ing terminated because of errors.\n"
         else:
-            term = "scons: done %sing targets.\n" % lc
+            term = f"scons: done {lc}ing targets.\n"
+
         return "scons: Reading SConscript files ...\n" + \
                read_str + \
                "scons: done reading SConscript files.\n" + \
-               "scons: %sing targets ...\n" % cap + \
+               f"scons: {cap}ing targets ...\n" + \
                build_str + \
                term
 
@@ -697,29 +734,53 @@ class TestSCons(TestCommon):
             result.extend(sorted(glob.glob(p)))
         return result
 
-    def unlink_sconsignfile(self, name='.sconsign.dblite'):
-        """
-        Delete sconsign file.
-        Note on python it seems to append .p3 to the file name so we take care of that
-        Parameters
-        ----------
-        name - expected name of sconsign file
+    def get_sconsignname(self):
+        """Get the scons database name used, and return both the prefix and full filename.
+        if the user left the options defaulted AND the default algorithm set by
+        SCons is md5, then set the database name to be the special default name
 
-        Returns
-        -------
-        None
+        otherwise, if it defaults to something like 'sha1' or the user explicitly
+        set 'md5' as the hash format, set the database name to .sconsign_<algorithm>
+        eg .sconsign_sha1, etc.
+
+        Returns:
+            a pair containing: the current dbname, the dbname.dblite filename
+        """
+        hash_format = get_hash_format()
+        current_hash_algorithm = get_current_hash_algorithm_used()
+        if hash_format is None and current_hash_algorithm == 'md5':
+            return ".sconsign"
+        else:
+            database_prefix=".sconsign_%s" % current_hash_algorithm
+            return database_prefix
+
+
+    def unlink_sconsignfile(self, name='.sconsign.dblite'):
+        """Delete the sconsign file.
+
+        Note on python it seems to append .p3 to the file name so we take
+        care of that.
+
+        TODO the above seems to not be an issue any more.
+
+        Args:
+            name: expected name of sconsign file
         """
         if sys.version_info[0] == 3:
             name += '.p3'
         self.unlink(name)
 
     def java_ENV(self, version=None):
-        """
+        """ Initialize JAVA SDK environment.
+
         Initialize with a default external environment that uses a local
         Java SDK in preference to whatever's found in the default PATH.
 
-        :param version: if set, match only that version
-        :return: the new env.
+        Args:
+            version: if set, match only that version
+
+        Returns:
+            the new env.
         """
         if not self.external:
             try:
@@ -764,11 +825,13 @@ class TestSCons(TestCommon):
         return None
 
     def java_where_includes(self, version=None):
-        """
-        Find include path needed for compiling java jni code.
+        """ Find include path needed for compiling java jni code.
 
-        :param version: if set, match only that version
-        :return: path to java headers
+        Args:
+            version: if set, match only that version
+
+        Returns:
+            path to java headers or None
         """
         import sys
 
@@ -801,66 +864,88 @@ class TestSCons(TestCommon):
             result.append(os.path.join(d, 'linux'))
         return result
 
-    def java_where_java_home(self, version=None):
-        """
-        Find path to what would be JAVA_HOME.
+    def java_where_java_home(self, version=None) -> str:
+        """ Find path to what would be JAVA_HOME.
 
         SCons does not read JAVA_HOME from the environment, so deduce it.
 
-        :param version: if set, match only that version
-        :return: path where JDK components live
+        Args:
+            version: if set, match only that version
+
+        Returns:
+            path where JDK components live
+            Bails out of the entire test (skip) if not found.
         """
         if sys.platform[:6] == 'darwin':
-            # osx 10.11, 10.12
+            # osx 10.11+
             home_tool = '/usr/libexec/java_home'
-            java_home = False
+            java_home = ''
             if os.path.exists(home_tool):
-                java_home = subprocess.check_output(home_tool).strip()
-                java_home = java_home.decode()
+                cp = sp.run(home_tool, stdout=sp.PIPE, stderr=sp.STDOUT)
+                if cp.returncode == 0:
+                    java_home = cp.stdout.decode().strip()
 
             if version is None:
                 if java_home:
                     return java_home
-                else:
-                    homes = ['/System/Library/Frameworks/JavaVM.framework/Home',
-                             # osx 10.10
-                             '/System/Library/Frameworks/JavaVM.framework/Versions/Current/Home']
-                    for home in homes:
-                        if os.path.exists(home):
-                            return home
-
+                for home in [
+                    '/System/Library/Frameworks/JavaVM.framework/Home',
+                    # osx 10.10
+                    '/System/Library/Frameworks/JavaVM.framework/Versions/Current/Home'
+                ]:
+                    if os.path.exists(home):
+                        return home
             else:
                 if java_home.find('jdk%s' % version) != -1:
                     return java_home
-                else:
-                    home = '/System/Library/Frameworks/JavaVM.framework/Versions/%s/Home' % version
-                    if not os.path.exists(home):
-                        # This works on OSX 10.10
-                        home = '/System/Library/Frameworks/JavaVM.framework/Versions/Current/'
+                for home in [
+                    '/System/Library/Frameworks/JavaVM.framework/Versions/%s/Home' % version,
+                    # osx 10.10
+                    '/System/Library/Frameworks/JavaVM.framework/Versions/Current/'
+                ]:
+                    if os.path.exists(home):
+                        return home
+            # if we fell through, make sure flagged as not found
+            home = ''
         else:
             jar = self.java_where_jar(version)
             home = os.path.normpath('%s/..' % jar)
-        if os.path.isdir(home):
+
+        if home and os.path.isdir(home):
             return home
-        print("Could not determine JAVA_HOME: %s is not a directory" % home)
-        self.fail_test()
 
-    def java_mac_check(self, where_java_bin, java_bin_name):
-        # on Mac there is a place holder java installed to start the java install process
-        # so we need to check the output in this case, more info here:
-        # http://anas.pk/2015/09/02/solution-no-java-runtime-present-mac-yosemite/
-        sp = subprocess.Popen([where_java_bin, "-version"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        stdout, stderr = sp.communicate()
-        sp.wait()
-        if "No Java runtime" in str(stderr):
-            self.skip_test("Could not find Java " + java_bin_name + ", skipping test(s).\n")
+        self.skip_test(
+            "Could not run Java: unable to detect valid JAVA_HOME, skipping test.\n",
+            from_fw=True,
+        )
 
-    def java_where_jar(self, version=None):
+    def java_mac_check(self, where_java_bin, java_bin_name) -> None:
+        """Extra check for Java on MacOS.
+
+        MacOS has a place holder java/javac, which fails with a detectable
+        error if Java is not actually installed, and works normally if it is.
+        Note msg has changed over time.
+
+        Bails out of the entire test (skip) if not found.
         """
-        Find java archiver jar.
+        cp = sp.run([where_java_bin, "-version"], stdout=sp.PIPE, stderr=sp.STDOUT)
+        if (
+            b"No Java runtime" in cp.stdout
+            or b"Unable to locate a Java Runtime" in cp.stdout
+        ):
+            self.skip_test(
+                "Could not find Java " + java_bin_name + ", skipping test.\n",
+                from_fw=True,
+            )
 
-        :param version: if set, match only that version
-        :return: path to jar
+    def java_where_jar(self, version=None) -> str:
+        """ Find java archiver jar.
+
+        Args:
+            version: if set, match only that version
+
+        Returns:
+            path to jar
         """
         ENV = self.java_ENV(version)
         if self.detect_tool('jar', ENV=ENV):
@@ -868,35 +953,39 @@ class TestSCons(TestCommon):
         else:
             where_jar = self.where_is('jar', ENV['PATH'])
         if not where_jar:
-            self.skip_test("Could not find Java jar, skipping test(s).\n")
+            self.skip_test("Could not find Java jar, skipping test(s).\n", from_fw=True)
         elif sys.platform == "darwin":
             self.java_mac_check(where_jar, 'jar')
 
         return where_jar
 
-    def java_where_java(self, version=None):
-        """
-        Find java executable.
+    def java_where_java(self, version=None) -> str:
+        """ Find java executable.
 
-        :param version: if set, match only that version
-        :return: path to the java rutime
+        Args:
+            version: if set, match only that version
+
+        Returns:
+            path to the java rutime
         """
         ENV = self.java_ENV(version)
         where_java = self.where_is('java', ENV['PATH'])
 
         if not where_java:
-            self.skip_test("Could not find Java java, skipping test(s).\n")
+            self.skip_test("Could not find Java java, skipping test(s).\n", from_fw=True)
         elif sys.platform == "darwin":
             self.java_mac_check(where_java, 'java')
 
         return where_java
 
-    def java_where_javac(self, version=None):
-        """
-        Find java compiler.
+    def java_where_javac(self, version=None) -> str:
+        """ Find java compiler.
 
-        :param version: if set, match only that version
-        :return: path to javac
+        Args:
+            version: if set, match only that version
+
+        Returns:
+            path to javac
         """
         ENV = self.java_ENV(version)
         if self.detect_tool('javac'):
@@ -904,7 +993,7 @@ class TestSCons(TestCommon):
         else:
             where_javac = self.where_is('javac', ENV['PATH'])
         if not where_javac:
-            self.skip_test("Could not find Java javac, skipping test(s).\n")
+            self.skip_test("Could not find Java javac, skipping test(s).\n", from_fw=True)
         elif sys.platform == "darwin":
             self.java_mac_check(where_javac, 'javac')
 
@@ -917,7 +1006,7 @@ class TestSCons(TestCommon):
             verf = 'javac %s' % version
             if self.stderr().find(verf) == -1 and self.stdout().find(verf) == -1:
                 fmt = "Could not find javac for Java version %s, skipping test(s).\n"
-                self.skip_test(fmt % version)
+                self.skip_test(fmt % version, from_fw=True)
         else:
             version_re = r'javac (\d*\.*\d)'
             m = re.search(version_re, self.stderr())
@@ -935,16 +1024,18 @@ class TestSCons(TestCommon):
                 self.javac_is_gcj = False
         return where_javac, version
 
-    def java_where_javah(self, version=None):
-        """
-        Find java header generation tool.
+    def java_where_javah(self, version=None) -> str:
+        """ Find java header generation tool.
 
         TODO issue #3347 since JDK10, there is no separate javah command,
         'javac -h' is used. We should not return a javah from a different
         installed JDK - how to detect and what to return in this case?
 
-        :param version: if set, match only that version
-        :return: path to javah
+        Args:
+            version: if set, match only that version
+
+        Returns:
+            path to javah
         """
         ENV = self.java_ENV(version)
         if self.detect_tool('javah'):
@@ -952,15 +1043,17 @@ class TestSCons(TestCommon):
         else:
             where_javah = self.where_is('javah', ENV['PATH'])
         if not where_javah:
-            self.skip_test("Could not find Java javah, skipping test(s).\n")
+            self.skip_test("Could not find Java javah, skipping test(s).\n", from_fw=True)
         return where_javah
 
-    def java_where_rmic(self, version=None):
-        """
-        Find java rmic tool.
+    def java_where_rmic(self, version=None) -> str:
+        """ Find java rmic tool.
 
-        :param version: if set, match only that version
-        :return: path to rmic
+        Args:
+            version: if set, match only that version
+
+        Returns:
+            path to rmic
         """
         ENV = self.java_ENV(version)
         if self.detect_tool('rmic'):
@@ -968,7 +1061,7 @@ class TestSCons(TestCommon):
         else:
             where_rmic = self.where_is('rmic', ENV['PATH'])
         if not where_rmic:
-            self.skip_test("Could not find Java rmic, skipping non-simulated test(s).\n")
+            self.skip_test("Could not find Java rmic, skipping non-simulated test(s).\n", from_fw=True)
         return where_rmic
 
     def java_get_class_files(self, dir):
@@ -1065,8 +1158,8 @@ void my_qt_symbol(const char *arg) {
 """)
 
         self.write([dir, 'lib', 'SConstruct'], r"""
-env = Environment()
 import sys
+env = Environment()
 if sys.platform == 'win32':
     env.StaticLibrary('myqt', 'my_qobject.cpp')
 else:
@@ -1088,20 +1181,20 @@ else:
         if isinstance(place, list):
             place = test.workpath(*place)
         self.write(place, """\
-if ARGUMENTS.get('noqtdir', 0): QTDIR=None
-else: QTDIR=r'%s'
-env = Environment(QTDIR = QTDIR,
-                  QT_LIB = r'%s',
-                  QT_MOC = r'%s',
-                  QT_UIC = r'%s',
-                  tools=['default','qt'])
+if ARGUMENTS.get('noqtdir', 0):
+    QTDIR = None
+else:
+    QTDIR = r'%s'
+env = Environment(
+    QTDIR=QTDIR, QT_LIB=r'%s', QT_MOC=r'%s', QT_UIC=r'%s', tools=['default', 'qt']
+)
 dup = 1
 if ARGUMENTS.get('variant_dir', 0):
     if ARGUMENTS.get('chdir', 0):
         SConscriptChdir(1)
     else:
         SConscriptChdir(0)
-    dup=int(ARGUMENTS.get('dup', 1))
+    dup = int(ARGUMENTS.get('dup', 1))
     if dup == 0:
         builddir = 'build_dup0'
         env['QT_DEBUG'] = 1
@@ -1135,26 +1228,27 @@ SConscript(sconscript)
         return 'COVERAGE_PROCESS_START' in os.environ or 'COVERAGE_FILE' in os.environ
 
     def skip_if_not_msvc(self, check_platform=True):
-        """ Check whether we are on a Windows platform and skip the
-            test if not. This check can be omitted by setting
-            check_platform to False.
-            Then, for a win32 platform, additionally check
-            whether we have a MSVC toolchain installed
-            in the system, and skip the test if none can be
-            found (=MinGW is the only compiler available).
+        """ Skip test if MSVC is not available.
+
+        Check whether we are on a Windows platform and skip the test if
+        not. This check can be omitted by setting check_platform to False.
+
+        Then, for a win32 platform, additionally check whether we have
+        an MSVC toolchain installed in the system, and skip the test if
+        none can be found (e.g. MinGW is the only compiler available).
         """
         if check_platform:
             if sys.platform != 'win32':
                 msg = "Skipping Visual C/C++ test on non-Windows platform '%s'\n" % sys.platform
-                self.skip_test(msg)
+                self.skip_test(msg, from_fw=True)
                 return
 
         try:
             import SCons.Tool.MSCommon as msc
             if not msc.msvc_exists():
                 msg = "No MSVC toolchain found...skipping test\n"
-                self.skip_test(msg)
-        except:
+                self.skip_test(msg, from_fw=True)
+        except Exception:
             pass
 
     def checkConfigureLogAndStdout(self, checks,
@@ -1162,29 +1256,24 @@ SConscript(sconscript)
                                    sconf_dir='.sconf_temp',
                                    sconstruct="SConstruct",
                                    doCheckLog=True, doCheckStdout=True):
-        """
+        """ Verify expected output from Configure.
+
         Used to verify the expected output from using Configure()
         via the contents of one or both of stdout or config.log file.
-        The checks, results, cached parameters all are zipped together
-        for use in comparing results.
+        If the algorithm does not succeed, the test is marked a fail
+        and this function does not return.
 
         TODO: Perhaps a better API makes sense?
 
-        Parameters
-        ----------
-        checks : list of ConfigCheckInfo tuples which specify
-        logfile : Name of the config log
-        sconf_dir : Name of the sconf dir
-        sconstruct : SConstruct file name
-        doCheckLog : check specified log file, defaults to true
-        doCheckStdout : Check stdout, defaults to true
-
-        Returns
-        -------
-
+        Args:
+            checks: list of ConfigCheckInfo tuples which specify
+            logfile: Name of the config log
+            sconf_dir: Name of the sconf dir
+            sconstruct: SConstruct file name
+            doCheckLog: check specified log file, defaults to true
+            doCheckStdout: Check stdout, defaults to true
         """
 
-
         try:
             ls = '\n'
             nols = '([^\n])'
@@ -1299,35 +1388,25 @@ SConscript(sconscript)
     def checkLogAndStdout(self, checks, results, cached,
                           logfile, sconf_dir, sconstruct,
                           doCheckLog=True, doCheckStdout=True):
-        """
+        """ Verify expected output from Configure.
+
         Used to verify the expected output from using Configure()
         via the contents of one or both of stdout or config.log file.
         The checks, results, cached parameters all are zipped together
-        for use in comparing results.
+        for use in comparing results. If the algorithm does not
+        succeed, the test is marked a fail and this function does not return.
 
         TODO: Perhaps a better API makes sense?
 
-        Parameters
-        ----------
-        checks : The Configure checks being run
-
-        results : The expected results for each check
-
-        cached  : If the corresponding check is expected to be cached
-
-        logfile : Name of the config log
-
-        sconf_dir : Name of the sconf dir
-
-        sconstruct : SConstruct file name
-
-        doCheckLog : check specified log file, defaults to true
-
-        doCheckStdout : Check stdout, defaults to true
-
-        Returns
-        -------
-
+        Args:
+            checks: The Configure checks being run
+            results: The expected results for each check
+            cached: If the corresponding check is expected to be cached
+            logfile: Name of the config log
+            sconf_dir: Name of the sconf dir
+            sconstruct: SConstruct file name
+            doCheckLog: check specified log file, defaults to true
+            doCheckStdout: Check stdout, defaults to true
         """
         try:
 
@@ -1371,10 +1450,10 @@ SConscript(sconscript)
                     for ext, flag in bld_desc:  # each file in TryBuild
                         if ext in ['.c', '.cpp']:
                             conf_filename = re.escape(os.path.join(sconf_dir, "conftest")) +\
-                                            r'_[a-z0-9]{32}_\d+%s' % re.escape(ext)
+                                            r'_[a-z0-9]{32,64}_\d+%s' % re.escape(ext)
                         elif ext == '':
                             conf_filename = re.escape(os.path.join(sconf_dir, "conftest")) +\
-                                            r'_[a-z0-9]{32}(_\d+_[a-z0-9]{32})?'
+                                            r'_[a-z0-9]{32,64}(_\d+_[a-z0-9]{32,64})?'
 
                         else:
                             # We allow the second hash group to be optional because
@@ -1386,7 +1465,7 @@ SConscript(sconscript)
                             # TODO: perhaps revisit and/or fix file naming for intermediate files in
                             #  Configure context logic
                             conf_filename = re.escape(os.path.join(sconf_dir, "conftest")) +\
-                                            r'_[a-z0-9]{32}_\d+(_[a-z0-9]{32})?%s' % re.escape(ext)
+                                            r'_[a-z0-9]{32,64}_\d+(_[a-z0-9]{32,64})?%s' % re.escape(ext)
 
                         if flag == self.NCR:
                             # NCR = Non Cached Rebuild
@@ -1460,16 +1539,18 @@ SConscript(sconscript)
                 print("-----------------------------------------------------")
                 self.fail_test()
 
-    def get_python_version(self):
-        """
-        Returns the Python version (just so everyone doesn't have to
-        hand-code slicing the right number of characters).
+    def get_python_version(self) -> str:
+        """ Returns the Python version.
+
+        Convenience function so everyone doesn't have to
+        hand-code slicing the right number of characters
         """
         # see also sys.prefix documentation
         return python_minor_version_string()
 
     def get_platform_python_info(self, python_h_required=False):
-        """
+        """Return information about Python.
+
         Returns a path to a Python executable suitable for testing on
         this platform and its associated include path, library path and
         library name.
@@ -1477,12 +1558,12 @@ SConscript(sconscript)
         If the Python executable or Python header (if required)
         is not found, the test is skipped.
 
-        Returns a tuple:
-            (path to python, include path, library path, library name)
+        Returns:
+            tuple: path to python, include path, library path, library name
         """
         python = os.environ.get('python_executable', self.where_is('python'))
         if not python:
-            self.skip_test('Can not find installed "python", skipping test.\n')
+            self.skip_test('Can not find installed "python", skipping test.\n', from_fw=True)
 
         # construct a program to run in the intended environment
         # in order to fetch the characteristics of that Python.
@@ -1491,12 +1572,9 @@ SConscript(sconscript)
             self.run(program=python, stdin="""\
 import sysconfig, sys, os.path
 py_ver = 'python%d%d' % sys.version_info[:2]
-# use distutils to help find include and lib path
-# TODO: PY3 fine to use sysconfig.get_config_var("INCLUDEPY")
 try:
-    import distutils.sysconfig
-    exec_prefix = distutils.sysconfig.EXEC_PREFIX
-    include = distutils.sysconfig.get_python_inc()
+    exec_prefix = sysconfig.get_config_var("exec_prefix")
+    include = sysconfig.get_config_var("INCLUDEPY")
     print(include)
     lib_path = os.path.join(exec_prefix, 'libs')
     if not os.path.exists(lib_path):
@@ -1542,7 +1620,7 @@ else:
 """)
         incpath, libpath, libname, python_h = self.stdout().strip().split('\n')
         if python_h == "False" and python_h_required:
-            self.skip_test('Can not find required "Python.h", skipping test.\n')
+            self.skip_test('Can not find required "Python.h", skipping test.\n', from_fw=True)
 
         return (python, incpath, libpath, libname)
 
@@ -1589,7 +1667,8 @@ else:
             waited = waited + 1.0
 
     def get_alt_cpp_suffix(self):
-        """
+        """Return alternate C++ file suffix.
+
         Many CXX tests have this same logic.
         They all needed to determine if the current os supports
         files with .C and .c as different files or not
@@ -1601,12 +1680,27 @@ else:
             alt_cpp_suffix = '.C'
         return alt_cpp_suffix
 
-    def platform_has_symlink(self):
+    def platform_has_symlink(self) -> bool:
+        """Retun an indication of whether symlink tests should be run.
+
+        Despite the name, we really mean "are they reliably usable"
+        rather than "do they exist" - basically the Windows case.
+        """
         if not hasattr(os, 'symlink') or sys.platform == 'win32':
             return False
         else:
             return True
 
+    def zipfile_contains(self, zipfilename, names):
+        """Returns True if zipfilename contains all the names, False otherwise."""
+        with zipfile.ZipFile(zipfilename, 'r') as zf:
+            return all(elem in zf.namelist() for elem in names)
+
+    def zipfile_files(self, fname):
+        """Returns all the filenames in zip file fname."""
+        with zipfile.ZipFile(fname, 'r') as zf:
+            return zf.namelist()
+
 
 class Stat:
     def __init__(self, name, units, expression, convert=None):
@@ -1681,7 +1775,7 @@ class TimeSCons(TestSCons):
         if 'verbose' not in kw and not self.calibrate:
             kw['verbose'] = True
 
-        TestSCons.__init__(self, *args, **kw)
+        super().__init__(*args, **kw)
 
         # TODO(sgk):    better way to get the script dir than sys.argv[0]
         self.test_dir = os.path.dirname(sys.argv[0])
@@ -1826,7 +1920,11 @@ class TimeSCons(TestSCons):
         # TODO(sgk):  allow the caller to specify the target (argument)
         # that must be up-to-date.
         self.add_timing_options(kw)
-        self.up_to_date(arguments='.', **kw)
+
+        # Build up regex for
+        # SConscript:/private/var/folders/ng/48pttrpj239fw5rmm3x65pxr0000gn/T/testcmd.12081.pk1bv5i5/SConstruct  took 533.646 ms
+        read_str = 'SConscript:.*\n'
+        self.up_to_date(arguments='.', read_str=read_str, **kw)
         sys.stdout.write(self.stdout())
         stats = self.collect_stats(self.stdout())
         # time-commands should always be 0.0 on a null build, because
@@ -1856,11 +1954,11 @@ class TimeSCons(TestSCons):
         --debug=memory and --debug=time options to have SCons report
         its own memory and timing statistics.
         """
-        self.startTime = time.time()
+        self.startTime = time.perf_counter()
         try:
             result = TestSCons.run(self, *args, **kw)
         finally:
-            self.endTime = time.time()
+            self.endTime = time.perf_counter()
         return result
 
     def copy_timing_configuration(self, source_dir, dest_dir):
@@ -1888,6 +1986,27 @@ class TimeSCons(TestSCons):
                 destination = source.replace(source_dir, dest_dir)
                 shutil.copy2(source, destination)
 
+    def up_to_date(self, arguments='.', read_str="", **kw):
+        """Asserts that all of the targets listed in arguments is
+        up to date, but does not make any assumptions on other targets.
+        This function is most useful in conjunction with the -n option.
+        Note: This custom version for timings tests does NOT escape
+              read_str.
+        """
+        s = ""
+        for arg in arguments.split():
+            s = s + "scons: `%s' is up to date.\n" % arg
+        kw['arguments'] = arguments
+        stdout = self.wrap_stdout(read_str="REPLACEME", build_str=s)
+        # Append '.*' so that timing output that comes after the
+        # up-to-date output is okay.
+        stdout = re.escape(stdout) + '.*'
+        stdout = stdout.replace('REPLACEME', read_str)
+        kw['stdout'] = stdout
+        kw['match'] = self.match_re_dotall
+        self.run(**kw)
+
+
 
 # In some environments, $AR will generate a warning message to stderr
 # if the library doesn't previously exist and is being created.  One
